package com.unissoft.config;

import com.unissoft.mqtt.inbound.MqttMessageReceiver;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.annotation.Bean;
import org.springframework.integration.annotation.ServiceActivator;
import org.springframework.integration.channel.DirectChannel;
import org.springframework.integration.config.EnableIntegration;
import org.springframework.integration.core.MessageProducer;
import org.springframework.integration.mqtt.core.MqttPahoClientFactory;
import org.springframework.integration.mqtt.inbound.MqttPahoMessageDrivenChannelAdapter;
import org.springframework.integration.mqtt.support.DefaultPahoMessageConverter;
import org.springframework.messaging.MessageChannel;
import org.springframework.messaging.MessageHandler;
import org.springframework.stereotype.Component;
import org.springframework.util.StringUtils;

/**
 * @author qingsong.han
 * @description: 消息订阅配置
 * @create 2023-02-14 14:08
 */
@Slf4j
@Component
@EnableIntegration
public class MqttInConfig {

    /*
     * mqtt入站消息处理配置
     */
    @Value("${mqtt.topics}")
    private String topics;
    @Value("${mqtt.timeout}")
    private long timeout;
    @Value("${mqtt.qos}")
    private int qos;
    @Value("${mqtt.clientId}")
    private String clientId;
    /*@Resource
    private BasicRemoteProtocolService remoteProtocolService;*/
    private final MqttMessageReceiver mqttMessageReceiver;

    @Autowired
    public MqttInConfig(MqttMessageReceiver mqttMessageReceiver) {
        this.mqttMessageReceiver = mqttMessageReceiver;
    }

    /**
     * mqtt消息入站通道，订阅消息后消息进入的通道。
     * 可创建多个入站通道，对应多个不同的消息生产者。
     *
     * @return MessageChannel
     */
    @Bean
    public MessageChannel mqttInputChannel() {
        return new DirectChannel();
    }

    /**
     * 对于当前应用来讲，接收的mqtt消息的生产者。将生产者绑定到mqtt入站通道，即通过入站通道进入的消息为生产者生产的消息。
     * 可创建多个消息生产者，对应多个不同的消息入站通道，同时生产者监听不同的topic
     *
     * @return MessageProducer
     */
    @Bean
    public MessageProducer channelInbound(MessageChannel mqttInputChannel, MqttPahoClientFactory factory) {
//        this.mqttInitDB();
        log.info("------------------db_data MessageProducer start-------------------: {}", topics);
        String[] topic = topics.trim().split(",");
        MqttPahoMessageDrivenChannelAdapter adapter = new MqttPahoMessageDrivenChannelAdapter(clientId, factory, topic);
        adapter.setCompletionTimeout(timeout);
        adapter.setConverter(new DefaultPahoMessageConverter());
        adapter.setOutputChannel(mqttInputChannel);
        return adapter;
    }

    /**
     * mqtt入站消息处理工具，对于指定消息入站通道接收到生产者生产的消息后处理消息的工具。
     *
     * @return
     */
    @Bean
    @ServiceActivator(inputChannel = MqttBaseConfig.CHANNEL_NAME_IN)
    public MessageHandler mqttMessageHandler() {
        return this.mqttMessageReceiver;
    }

    /**
     * 数据库配置订阅消息
     */
    /*private void mqttInitDB() {
        mqttAddTopic(remoteProtocolService.getByTopics());
    }*/

    /**
     * 添加配置topic
     *
     * @param newTopic 新topics
     */
    private void mqttAddTopic(String newTopic) {
        if (StringUtils.isEmpty(newTopic))
            topics = "mqtt_default_topic";
        else
            topics = newTopic;
    }

}
